"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMessageBusPorts = exports.connectToMessageBus = exports.connectToMessageBusWithRetries = exports.sendAndAwait = exports.broadcastAndAwaitFirst = exports.broadcastAndDisregard = exports.createMessage = exports.startWebSocketServer = exports.base64ToJson = exports.jsonToBase64 = void 0;
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const promise_any_1 = __importDefault(require("promise.any"));
const delay_1 = __importDefault(require("delay"));
const axios_1 = __importDefault(require("axios"));
promise_any_1.default.shim();
/**
 * Convert any JS object or value to a base64 representation of it
 */
const jsonToBase64 = (json) => {
    const stringifiedJson = JSON.stringify(json);
    const buffer = Buffer.from(stringifiedJson);
    const base64 = buffer.toString('base64');
    return base64;
};
exports.jsonToBase64 = jsonToBase64;
/**
 * Convert the base64 representation of a JS object or value to its JS representation
 * @dev This is the reverse of `jsonToBase64` and is not expected to work with other base64 formats
 */
const base64ToJson = (base64) => {
    const buffer = Buffer.from(base64, 'base64');
    const stringifiedJson = buffer.toString('utf8');
    const json = JSON.parse(stringifiedJson);
    return json;
};
exports.base64ToJson = base64ToJson;
/**
 * Starts a websocket server and waits for it to be opened
 * @dev If you need to attach event listeners *before* the server connection opens,
 * do not use this function since it resolves *after* the connection is opened
 */
const startWebSocketServer = (options) => {
    return new Promise((resolve) => {
        const server = new isomorphic_ws_1.default.Server(options, () => resolve(server));
    });
};
exports.startWebSocketServer = startWebSocketServer;
const createMessage = (type, payload) => {
    const id = Math.random();
    return { id, type, payload };
};
exports.createMessage = createMessage;
/**
 * Broadcast a message to multiple websocket connections and disregard them
 */
const broadcastAndDisregard = (sockets, message) => {
    const encodedMessage = (0, exports.jsonToBase64)(message);
    sockets.forEach((socket) => {
        socket.send(encodedMessage);
    });
};
exports.broadcastAndDisregard = broadcastAndDisregard;
/**
 * Broadcast a message to multuple websocket connections and return the first response
 */
const broadcastAndAwaitFirst = (sockets, message) => __awaiter(void 0, void 0, void 0, function* () {
    const promises = sockets.map((socket) => (0, exports.sendAndAwait)(socket, message));
    const result = yield Promise.any(promises);
    return result;
});
exports.broadcastAndAwaitFirst = broadcastAndAwaitFirst;
/**
 * Send a message to a websocket connection and await a matching response
 * @dev Responses are matched by looking at received messages that match the ID of the sent message
 */
const sendAndAwait = (socket, message) => {
    return new Promise((resolve, reject) => {
        socket.addEventListener('message', (event) => {
            if (typeof event.data !== 'string') {
                event.data = event.data.toString();
            }
            const response = (0, exports.base64ToJson)(event.data);
            if (response.id !== message.id)
                return;
            resolve(response);
        });
        // TODO: Need to check that the error corresponds to the sent message?
        socket.addEventListener('error', (event) => {
            reject(event.error);
        });
        socket.addEventListener('close', (event) => {
            reject(new Error(`Socket connection closed with code '${event.code}' and reason '${event.reason}'`));
        });
        const encodedMessage = (0, exports.jsonToBase64)(message);
        socket.send(encodedMessage);
    });
};
exports.sendAndAwait = sendAndAwait;
const connectToMessageBusWithRetries = (port, host = 'localhost', retries = 50) => __awaiter(void 0, void 0, void 0, function* () {
    let error = new Error();
    for (let tryCount = 0; tryCount < retries; tryCount += 1) {
        try {
            return yield (0, exports.connectToMessageBus)(port, host);
        }
        catch (e) {
            error = e;
            yield (0, delay_1.default)(1000);
        }
    }
    throw error;
});
exports.connectToMessageBusWithRetries = connectToMessageBusWithRetries;
const connectToMessageBus = (port, host = 'localhost') => {
    const socket = new isomorphic_ws_1.default(`ws://${host}:${port}`);
    return new Promise((resolve, reject) => {
        socket.addEventListener('open', () => resolve(socket));
        socket.addEventListener('error', (event) => reject(event.error));
    });
};
exports.connectToMessageBus = connectToMessageBus;
const getMessageBusPorts = (dashboardPort, dashboardHost = 'localhost', retries = 5) => __awaiter(void 0, void 0, void 0, function* () {
    for (let tryCount = 0; tryCount < retries; tryCount += 1) {
        try {
            const { data } = yield axios_1.default.get(`http://${dashboardHost}:${dashboardPort}/ports`);
            return data;
        }
        catch (e) {
            yield (0, delay_1.default)(1000);
        }
    }
    throw new Error(`[ERROR]: Unable to connect to dashboard at http://${dashboardHost}:${dashboardPort}/ports`);
});
exports.getMessageBusPorts = getMessageBusPorts;
//# sourceMappingURL=utils.js.map